@@ -1,4 +1,4 @@
-/* $Id: logout.c,v 1.206 2006/01/23 14:05:59 fpeters Exp $
+/* $Id: logout.c,v 1.221 2007/01/08 13:13:49 fpeters Exp $
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -75,6 +75,7 @@ lasso_logout_build_request_msg(LassoLogout *logout)
 	g_return_val_if_fail(LASSO_IS_LOGOUT(logout), LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
 
 	profile = LASSO_PROFILE(logout);
+	lasso_profile_clean_msg_info(profile);
 
 	if (profile->remote_providerID == NULL) {
 		/* this means lasso_logout_init_request was not called before */
@@ -120,7 +121,7 @@ lasso_logout_build_request_msg(LassoLogout *logout)
 			return critical_error(LASSO_PROFILE_ERROR_BUILDING_QUERY_FAILED);
 		}
 		/* build the msg_url */
-		profile->msg_url = g_strdup_printf("%s?%s", url, query);
+		profile->msg_url = lasso_concat_url_query(url, query);
 		g_free(url);
 		g_free(query);
 		profile->msg_body = NULL;
@@ -167,6 +168,7 @@ lasso_logout_build_response_msg(LassoLogout *logout)
 	g_return_val_if_fail(LASSO_IS_LOGOUT(logout), LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
 
 	profile = LASSO_PROFILE(logout);
+	lasso_profile_clean_msg_info(profile);
 
 	IF_SAML2(profile) {
 		return lasso_saml20_logout_build_response_msg(logout);
@@ -231,7 +233,7 @@ lasso_logout_build_response_msg(LassoLogout *logout)
 			g_free(url);
 			return critical_error(LASSO_PROFILE_ERROR_BUILDING_QUERY_FAILED);
 		}
-		profile->msg_url = g_strdup_printf("%s?%s", url, query);
+		profile->msg_url = lasso_concat_url_query(url, query);
 		profile->msg_body = NULL;
 		g_free(url);
 		g_free(query);
@@ -294,7 +296,7 @@ lasso_logout_get_next_providerID(LassoLogout *logout)
  * @request_method: if set, then it get the protocol profile in metadata
  *     corresponding of this HTTP request method.
  *
- * Initializes a new lib:LogoutRequest.
+ * Initializes a new SLO request.
  * 
  * Return value: 0 on success; or a negative value otherwise.
  **/
@@ -307,6 +309,7 @@ lasso_logout_init_request(LassoLogout *logout, char *remote_providerID,
 	LassoSamlNameIdentifier *nameIdentifier;
 	LassoNode *assertion_n, *name_identifier_n;
 	LassoSamlAssertion *assertion;
+	LassoSamlSubjectStatementAbstract *subject_statement;
 	LassoFederation   *federation = NULL;
 	gboolean           is_http_redirect_get_method = FALSE;
 	LassoSession *session;
@@ -347,8 +350,7 @@ lasso_logout_init_request(LassoLogout *logout, char *remote_providerID,
 	/* get assertion */
 	assertion_n = lasso_session_get_assertion(session, profile->remote_providerID);
 	if (LASSO_IS_SAML_ASSERTION(assertion_n) == FALSE) {
-		message(G_LOG_LEVEL_CRITICAL, "Assertion not found");
-		return LASSO_ERROR_UNDEFINED;
+		return critical_error(LASSO_PROFILE_ERROR_MISSING_ASSERTION);
 	}
 	
 	assertion = LASSO_SAML_ASSERTION(assertion_n);
@@ -361,12 +363,21 @@ lasso_logout_init_request(LassoLogout *logout, char *remote_providerID,
 			session_index = g_strdup(as->SessionIndex);
 	}
 
-
 	/* if format is one time, then get name identifier from assertion,
 	   else get name identifier from federation */
-	nameIdentifier = LASSO_SAML_SUBJECT_STATEMENT_ABSTRACT(
-			assertion->AuthenticationStatement)->Subject->NameIdentifier;
-	if (strcmp(nameIdentifier->Format, LASSO_LIB_NAME_IDENTIFIER_FORMAT_ONE_TIME) != 0) {
+	subject_statement = NULL;
+	nameIdentifier = NULL;
+	if (LASSO_IS_SAML_SUBJECT_STATEMENT_ABSTRACT(assertion->AuthenticationStatement)) {
+		subject_statement = LASSO_SAML_SUBJECT_STATEMENT_ABSTRACT(
+				assertion->AuthenticationStatement);
+		if (subject_statement && subject_statement->Subject) {
+			nameIdentifier = subject_statement->Subject->NameIdentifier;
+		}
+	}
+
+
+	if (nameIdentifier && strcmp(nameIdentifier->Format,
+				LASSO_LIB_NAME_IDENTIFIER_FORMAT_ONE_TIME) != 0) {
 		if (LASSO_IS_IDENTITY(profile->identity) == FALSE) {
 			return critical_error(LASSO_PROFILE_ERROR_IDENTITY_NOT_FOUND);
 		}
@@ -476,7 +487,7 @@ lasso_logout_init_request(LassoLogout *logout, char *remote_providerID,
  * @logout: a #LassoLogout
  * @request_msg: the logout request message
  * 
- * Processes a lib:LogoutRequest message.  Rebuilds a request object from the
+ * Processes a SLO LogoutRequest message.  Rebuilds a request object from the
  * message and optionally verifies its signature.
  *
  * Return value: 0 on success; or a negative value otherwise.
@@ -488,8 +499,8 @@ lasso_logout_process_request_msg(LassoLogout *logout, char *request_msg)
 	LassoProvider *remote_provider;
 	LassoMessageFormat format;
 
-	g_return_val_if_fail(LASSO_IS_LOGOUT(logout), -1);
-	g_return_val_if_fail(request_msg != NULL, -1);
+	g_return_val_if_fail(LASSO_IS_LOGOUT(logout), LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(request_msg != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(logout);
 
@@ -503,8 +514,15 @@ lasso_logout_process_request_msg(LassoLogout *logout, char *request_msg)
 		return critical_error(LASSO_PROFILE_ERROR_INVALID_MSG);
 	}
 
-	remote_provider = g_hash_table_lookup(profile->server->providers,
+	if (profile->remote_providerID) {
+		g_free(profile->remote_providerID);
+	}
+
+	profile->remote_providerID = g_strdup(
 			LASSO_LIB_LOGOUT_REQUEST(profile->request)->ProviderID);
+
+	remote_provider = g_hash_table_lookup(profile->server->providers,
+			profile->remote_providerID);
 	if (LASSO_IS_PROVIDER(remote_provider) == FALSE) {
 		return critical_error(LASSO_SERVER_ERROR_PROVIDER_NOT_FOUND);
 	}
@@ -558,12 +576,12 @@ lasso_logout_process_response_msg(LassoLogout *logout, gchar *response_msg)
 	int rc;
 
 	g_return_val_if_fail(LASSO_IS_LOGOUT(logout), LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(response_msg != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(response_msg != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(logout);
 
 	IF_SAML2(profile) {
-		return lasso_saml20_process_response_msg(logout, response_msg);
+		return lasso_saml20_logout_process_response_msg(logout, response_msg);
 	}
 
 	/* before verify if profile->response is set */
@@ -613,8 +631,7 @@ lasso_logout_process_response_msg(LassoLogout *logout, gchar *response_msg)
 
 	if (response->Status == NULL || response->Status->StatusCode == NULL
 			|| response->Status->StatusCode->Value == NULL) {
-		message(G_LOG_LEVEL_CRITICAL, "No Status in LogoutResponse !");
-		return LASSO_ERROR_UNDEFINED;
+		return critical_error(LASSO_PROFILE_ERROR_MISSING_STATUS_CODE);
 	}
 	statusCodeValue = response->Status->StatusCode->Value;
 
@@ -644,7 +661,7 @@ lasso_logout_process_response_msg(LassoLogout *logout, gchar *response_msg)
 				g_free(url);
 				return critical_error(LASSO_PROFILE_ERROR_BUILDING_QUERY_FAILED);
 			}
-			profile->msg_url = g_strdup_printf("%s?%s", url, query);
+			profile->msg_url = lasso_concat_url_query(url, query);
 			g_free(url);
 			g_free(query);
 			profile->msg_body = NULL;
@@ -657,12 +674,22 @@ lasso_logout_process_response_msg(LassoLogout *logout, gchar *response_msg)
 		}
 		if (strcmp(statusCodeValue, LASSO_SAML_STATUS_CODE_REQUEST_DENIED) == 0) {
 			/* assertion no longer on idp so removing it locally too */
+			message(G_LOG_LEVEL_WARNING, "SP answer is request denied");
 			lasso_session_remove_assertion(
 					profile->session, profile->remote_providerID);
 			return LASSO_LOGOUT_ERROR_REQUEST_DENIED;
 		}
+		if (strcmp(statusCodeValue,
+					LASSO_LIB_STATUS_CODE_FEDERATION_DOES_NOT_EXIST) == 0) {
+			/* how could this happen ?  probably error in SP */
+			/* let's remove the assertion nevertheless */
+			message(G_LOG_LEVEL_WARNING, "SP answer is federation does not exist");
+			lasso_session_remove_assertion(
+					profile->session, profile->remote_providerID);
+			return LASSO_LOGOUT_ERROR_FEDERATION_NOT_FOUND;
+		}
 		message(G_LOG_LEVEL_CRITICAL, "Status code is not success : %s", statusCodeValue);
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_PROFILE_ERROR_STATUS_NOT_SUCCESS;
 	}
 
 	/* LogoutResponse status code value is ok */
@@ -727,7 +754,7 @@ lasso_logout_process_response_msg(LassoLogout *logout, gchar *response_msg)
  **/
 gint lasso_logout_reset_providerID_index(LassoLogout *logout)
 {
-	g_return_val_if_fail(LASSO_IS_LOGOUT(logout), -1);
+	g_return_val_if_fail(LASSO_IS_LOGOUT(logout), LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
 	lasso_session_init_provider_ids(LASSO_PROFILE(logout)->session);
 	logout->providerID_index = 0;
 	return 0;
@@ -782,6 +809,10 @@ lasso_logout_validate_request(LassoLogout *logout)
 	if (LASSO_IS_LIB_LOGOUT_REQUEST(profile->request) == FALSE)
 		return LASSO_PROFILE_ERROR_MISSING_REQUEST;
 
+	if (profile->remote_providerID) {
+		g_free(profile->remote_providerID);
+	}
+
 	profile->remote_providerID = g_strdup(
 			LASSO_LIB_LOGOUT_REQUEST(profile->request)->ProviderID);
 
@@ -837,10 +868,10 @@ lasso_logout_validate_request(LassoLogout *logout)
 
 	/* verify authentication */
 	assertion_n = lasso_session_get_assertion(profile->session, profile->remote_providerID);
-	if (assertion_n == NULL) {
+	if (LASSO_IS_SAML_ASSERTION(assertion_n) == FALSE) {
 		message(G_LOG_LEVEL_WARNING, "%s has no assertion", profile->remote_providerID);
 		lasso_profile_set_response_status(profile, LASSO_SAML_STATUS_CODE_REQUEST_DENIED);
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_PROFILE_ERROR_MISSING_ASSERTION;
 	}
 
 	assertion = LASSO_SAML_ASSERTION(assertion_n);
@@ -866,7 +897,7 @@ lasso_logout_validate_request(LassoLogout *logout)
 					profile->remote_providerID);
 			lasso_profile_set_response_status(profile,
 					LASSO_LIB_STATUS_CODE_FEDERATION_DOES_NOT_EXIST);
-			return LASSO_ERROR_UNDEFINED;
+			return LASSO_LOGOUT_ERROR_FEDERATION_NOT_FOUND;
 		}
 	}
 
@@ -920,6 +951,8 @@ static struct XmlSnippet schema_snippets[] = {
 		G_STRUCT_OFFSET(LassoLogout, initial_response) },
 	{ "InitialRemoteProviderID", SNIPPET_CONTENT,
 		G_STRUCT_OFFSET(LassoLogout, initial_remote_providerID) },
+	{ "InitialHttpRequestMethod", SNIPPET_CONTENT | SNIPPET_INTEGER,
+		G_STRUCT_OFFSET(LassoLogout, initial_http_request_method) },
 	/* "ProviderIdIndex" must not be dumped (since apps assume to get
 	 * it back to 0 after a restore from dump) (maybe this behaviour should
 	 * be fixed)
@@ -940,8 +973,9 @@ check_soap_support(gchar *key, LassoProvider *provider, LassoProfile *profile)
 		return; /* original service provider (initiated logout) */
 
 	assertion_n = lasso_session_get_assertion(profile->session, provider->ProviderID);
-	if (assertion_n == NULL)
+	if (LASSO_IS_SAML_ASSERTION(assertion_n) == FALSE) {
 		return; /* not authenticated with this provider */
+	}
 	assertion = LASSO_SAML_ASSERTION(assertion_n);
 
 	supported_profiles = lasso_provider_get_metadata_list(provider,
@@ -1094,9 +1128,13 @@ lasso_logout_new_from_dump(LassoServer *server, const char *dump)
 	LassoLogout *logout;
 	xmlDoc *doc;
 
+	if (dump == NULL)
+		return NULL;
+
 	logout = lasso_logout_new(g_object_ref(server));
 	doc = xmlParseMemory(dump, strlen(dump));
 	init_from_xml(LASSO_NODE(logout), xmlDocGetRootElement(doc)); 
+	xmlFreeDoc(doc);
 
 	return logout;
 }
