@@ -1,4 +1,4 @@
-/* $Id: name_identifier_mapping.c,v 1.59 2006/01/23 15:30:00 fpeters Exp $
+/* $Id: name_identifier_mapping.c,v 1.65 2007/01/05 13:40:07 fpeters Exp $
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -57,9 +57,11 @@ lasso_name_identifier_mapping_build_request_msg(LassoNameIdentifierMapping *mapp
 	LassoProfile *profile;
 	LassoProvider *remote_provider;
 
-	g_return_val_if_fail(LASSO_IS_NAME_IDENTIFIER_MAPPING(mapping), -1);
+	g_return_val_if_fail(LASSO_IS_NAME_IDENTIFIER_MAPPING(mapping),
+			LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(mapping);
+	lasso_profile_clean_msg_info(profile);
 
 	if (profile->remote_providerID == NULL) {
 		/* this means lasso_logout_init_request was not called before */
@@ -75,7 +77,7 @@ lasso_name_identifier_mapping_build_request_msg(LassoNameIdentifierMapping *mapp
 
 	if (remote_provider->role != LASSO_PROVIDER_ROLE_IDP) {
 		message(G_LOG_LEVEL_CRITICAL, "Build request msg method is forbidden at IDP");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_NAME_IDENTIFIER_MAPPING_ERROR_FORBIDDEN_CALL_ON_THIS_SIDE;
 	}
 
 	profile->msg_url = lasso_provider_get_metadata_one(remote_provider, "SoapEndpoint");
@@ -126,9 +128,11 @@ lasso_name_identifier_mapping_build_response_msg(LassoNameIdentifierMapping *map
 	LassoProfile *profile;
 	LassoProvider *remote_provider;
 
-	g_return_val_if_fail(LASSO_IS_NAME_IDENTIFIER_MAPPING(mapping), -1);
+	g_return_val_if_fail(LASSO_IS_NAME_IDENTIFIER_MAPPING(mapping),
+			LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(mapping);
+	lasso_profile_clean_msg_info(profile);
 
 	if (profile->remote_providerID == NULL) {
 		/* this means lasso_logout_init_request was not called before */
@@ -143,7 +147,7 @@ lasso_name_identifier_mapping_build_response_msg(LassoNameIdentifierMapping *map
 
 	if (remote_provider->role != LASSO_PROVIDER_ROLE_SP) {
 		message(G_LOG_LEVEL_CRITICAL, "Build response msg method is forbidden at SP");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_NAME_IDENTIFIER_MAPPING_ERROR_FORBIDDEN_CALL_ON_THIS_SIDE;
 	}
 
 	/* verify the provider type is a service provider type */
@@ -197,8 +201,8 @@ lasso_name_identifier_mapping_init_request(LassoNameIdentifierMapping *mapping,
 
 	g_return_val_if_fail(LASSO_IS_NAME_IDENTIFIER_MAPPING(mapping),
 			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(targetNamespace != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(remote_providerID != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(targetNamespace != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
+	g_return_val_if_fail(remote_providerID != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(mapping);
 
@@ -218,7 +222,7 @@ lasso_name_identifier_mapping_init_request(LassoNameIdentifierMapping *mapping,
 	}
 	if (remote_provider->role != LASSO_PROVIDER_ROLE_IDP) {
 		message(G_LOG_LEVEL_CRITICAL, "Init request method is forbidden for an IDP");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_NAME_IDENTIFIER_MAPPING_ERROR_FORBIDDEN_CALL_ON_THIS_SIDE;
 	}
 
 	/* get federation */
@@ -287,7 +291,7 @@ lasso_name_identifier_mapping_process_request_msg(LassoNameIdentifierMapping *ma
 
 	g_return_val_if_fail(LASSO_IS_NAME_IDENTIFIER_MAPPING(mapping),
 			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(request_msg != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(request_msg != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(mapping);
 
@@ -345,12 +349,13 @@ lasso_name_identifier_mapping_process_response_msg(LassoNameIdentifierMapping *m
 	LassoProfile  *profile;
 	LassoProvider *remote_provider;
 	LassoMessageFormat format;
+	LassoLibNameIdentifierMappingResponse *response;
 	int rc;
 	char *statusCodeValue;
 
 	g_return_val_if_fail(LASSO_IS_NAME_IDENTIFIER_MAPPING(mapping),
 			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(response_msg != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(response_msg != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(mapping);
 
@@ -360,8 +365,9 @@ lasso_name_identifier_mapping_process_response_msg(LassoNameIdentifierMapping *m
 		return critical_error(LASSO_PROFILE_ERROR_INVALID_MSG);
 	}
 
-	remote_provider = g_hash_table_lookup(profile->server->providers,
-			LASSO_LIB_NAME_IDENTIFIER_MAPPING_RESPONSE(profile->response)->ProviderID);
+	response = LASSO_LIB_NAME_IDENTIFIER_MAPPING_RESPONSE(profile->response);
+
+	remote_provider = g_hash_table_lookup(profile->server->providers, response->ProviderID);
 	if (LASSO_IS_PROVIDER(remote_provider) == FALSE) {
 		return critical_error(LASSO_SERVER_ERROR_PROVIDER_NOT_FOUND);
 	}
@@ -369,16 +375,25 @@ lasso_name_identifier_mapping_process_response_msg(LassoNameIdentifierMapping *m
 	/* verify signature */
 	rc = lasso_provider_verify_signature(remote_provider, response_msg, "ResponseID", format);
 
-	statusCodeValue = LASSO_LIB_NAME_IDENTIFIER_MAPPING_RESPONSE(
-			profile->response)->Status->StatusCode->Value;
-	if (strcmp(statusCodeValue, LASSO_SAML_STATUS_CODE_SUCCESS) != 0) {
-		message(G_LOG_LEVEL_CRITICAL, "%s", statusCodeValue);
-		return LASSO_ERROR_UNDEFINED;
+	if (response->Status == NULL || response->Status->StatusCode == NULL) {
+		return LASSO_PROFILE_ERROR_MISSING_STATUS_CODE;
 	}
 
+	statusCodeValue = response->Status->StatusCode->Value;
+	if (statusCodeValue == NULL || strcmp(statusCodeValue,
+				LASSO_SAML_STATUS_CODE_SUCCESS) != 0) {
+		return LASSO_PROFILE_ERROR_STATUS_NOT_SUCCESS;
+	}
+
+	
 	/* Set the target name identifier */
-	mapping->targetNameIdentifier = g_strdup(LASSO_LIB_NAME_IDENTIFIER_MAPPING_REQUEST(
-			profile->request)->NameIdentifier->content);
+	if (LASSO_LIB_NAME_IDENTIFIER_MAPPING_REQUEST(profile->request)->NameIdentifier) {
+		mapping->targetNameIdentifier = g_strdup(LASSO_LIB_NAME_IDENTIFIER_MAPPING_REQUEST(
+					profile->request)->NameIdentifier->content);
+	} else {
+		mapping->targetNameIdentifier = NULL;
+		return LASSO_NAME_IDENTIFIER_MAPPING_ERROR_MISSING_TARGET_IDENTIFIER;
+	}
 
 	return rc;
 }
@@ -420,13 +435,13 @@ lasso_name_identifier_mapping_validate_request(LassoNameIdentifierMapping *mappi
 
 	if (remote_provider->role != LASSO_PROVIDER_ROLE_SP) {
 		message(G_LOG_LEVEL_CRITICAL, "Build request msg method is forbidden at SP");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_NAME_IDENTIFIER_MAPPING_ERROR_FORBIDDEN_CALL_ON_THIS_SIDE;
 	}
 
 	/* verify request attribute of mapping is a name identifier mapping request */
 	if (LASSO_IS_LIB_NAME_IDENTIFIER_MAPPING_REQUEST(profile->request) == FALSE) {
 		message(G_LOG_LEVEL_CRITICAL, "Invalid NameIdentifierMappingRequest");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_PROFILE_ERROR_MISSING_REQUEST;
 	}
 
 	if (profile->http_request_method != LASSO_HTTP_METHOD_SOAP) {
@@ -473,21 +488,19 @@ lasso_name_identifier_mapping_validate_request(LassoNameIdentifierMapping *mappi
 	if (nameIdentifier == NULL) {
 		lasso_profile_set_response_status(profile,
 				LASSO_LIB_STATUS_CODE_UNKNOWN_PRINCIPAL);
-		message(G_LOG_LEVEL_CRITICAL, "Name identifier of federation not found");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_PROFILE_ERROR_NAME_IDENTIFIER_NOT_FOUND;
 	}
 
 	/* get the federation of the target name space and his name identifier */
 	if (request->TargetNamespace == NULL) {
-		message(G_LOG_LEVEL_CRITICAL, "Target name space not found");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_NAME_IDENTIFIER_MAPPING_ERROR_MISSING_TARGET_NAMESPACE;
 	}
 	federation = g_hash_table_lookup(profile->identity->federations, request->TargetNamespace);
 	if (LASSO_IS_FEDERATION(federation) == FALSE) {
 		lasso_profile_set_response_status(profile,
 				LASSO_LIB_STATUS_CODE_FEDERATION_DOES_NOT_EXIST);
 		message(G_LOG_LEVEL_CRITICAL, "Target name space federation not found");
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_PROFILE_ERROR_FEDERATION_NOT_FOUND;
 	}
 
 	targetNameIdentifier = LASSO_SAML_NAME_IDENTIFIER(federation->remote_nameIdentifier);
@@ -500,7 +513,7 @@ lasso_name_identifier_mapping_validate_request(LassoNameIdentifierMapping *mappi
 				"Name identifier for target name space federation not found");
 		lasso_profile_set_response_status(profile,
 				LASSO_LIB_STATUS_CODE_FEDERATION_DOES_NOT_EXIST);
-		return LASSO_ERROR_UNDEFINED;
+		return LASSO_PROFILE_ERROR_NAME_IDENTIFIER_NOT_FOUND;
 	}
 
 	LASSO_LIB_NAME_IDENTIFIER_MAPPING_RESPONSE(profile->response)->NameIdentifier =
