@@ -1,4 +1,4 @@
-/* $Id: provider.h,v 1.23 2006/02/21 09:51:49 fpeters Exp $ 
+/* $Id: provider.h,v 1.29 2006/12/27 15:04:00 fpeters Exp $ 
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -30,6 +30,7 @@ extern "C" {
 #endif /* __cplusplus */ 
 
 #include <lasso/xml/xml.h>
+#include <lasso/xml/xml_enc.h>
 
 #define LASSO_TYPE_PROVIDER (lasso_provider_get_type())
 #define LASSO_PROVIDER(obj) \
@@ -115,10 +116,10 @@ typedef enum {
 
 /**
  * LassoProtocolConformance:
- * LASSO_PROTOCOL_LIBERTY_1_0: Liberty ID-FF 1.0
- * LASSO_PROTOCOL_LIBERTY_1_1: Liberty ID-FF 1.1
- * LASSO_PROTOCOL_LIBERTY_1_2: Liberty ID-FF 1.2 / ID-WSF 1.0
- * LASSO_PROTOCOL_SAML_2_0: SAML 2.0
+ * @LASSO_PROTOCOL_LIBERTY_1_0: Liberty ID-FF 1.0
+ * @LASSO_PROTOCOL_LIBERTY_1_1: Liberty ID-FF 1.1
+ * @LASSO_PROTOCOL_LIBERTY_1_2: Liberty ID-FF 1.2 / ID-WSF 1.0
+ * @LASSO_PROTOCOL_SAML_2_0: SAML 2.0
  *
  * Provider protocol conformance.
  **/
@@ -130,6 +131,21 @@ typedef enum {
 } LassoProtocolConformance;
 
 
+/**
+ * LassoEncryptionMode:
+ * @LASSO_ENCRYPTION_MODE_NONE: Encrypt nothing
+ * @LASSO_ENCRYPTION_MODE_NAMEID: Encrypt NameIDs
+ * @LASSO_ENCRYPTION_MODE_ASSERTION : Encrypt Assertions
+ *
+ * Encryption mode.
+ **/
+typedef enum {
+	LASSO_ENCRYPTION_MODE_NONE,
+	LASSO_ENCRYPTION_MODE_NAMEID,
+	LASSO_ENCRYPTION_MODE_ASSERTION,
+} LassoEncryptionMode;
+
+
 struct _LassoProvider {
 	LassoNode parent;
 
@@ -176,6 +192,12 @@ LASSO_EXPORT xmlNode* lasso_provider_get_organization(LassoProvider *provider);
 LASSO_EXPORT LassoProtocolConformance lasso_provider_get_protocol_conformance(
 		LassoProvider *provider);
 
+LASSO_EXPORT void lasso_provider_set_encryption_mode(LassoProvider *provider,
+		LassoEncryptionMode encryption_mode);
+
+LASSO_EXPORT void lasso_provider_set_encryption_sym_key_type(LassoProvider *provider,
+		LassoEncryptionSymKeyType encryption_sym_key_type);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
