@@ -1,4 +1,4 @@
-/* $Id: authentication.c,v 1.10 2006/03/04 15:50:31 fpeters Exp $ 
+/* $Id: authentication.c,v 1.19 2007/01/06 22:53:19 fpeters Exp $ 
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -34,8 +34,6 @@ static sasl_callback_t lasso_sasl_callbacks[5];
 static int
 lasso_sasl_cb_log(void* context, int priority, const char* message)
 {
-	printf("SASL lasso_sasl_cb_log() : %s\n", message);
-
 	return SASL_OK;
 }
 
@@ -64,12 +62,12 @@ lasso_sasl_cb_pass(sasl_conn_t* conn, void* context, int id, sasl_secret_t** pse
 	static sasl_secret_t *s;
 	LassoUserAccount *account;
 	int ret = SASL_FAIL;
-   
+
 	account = (LassoUserAccount *)context;
 	if (account != NULL && account->password != NULL) {
 		s = (sasl_secret_t*) g_malloc0(sizeof(sasl_secret_t) + strlen(account->password));
 
-		strcpy(s->data, account->password);
+		strcpy((char*)s->data, account->password);
 		s->len = strlen(account->password);
 
 		*psecret = s;
@@ -112,9 +110,9 @@ lasso_authentication_client_start(LassoAuthentication *authentication)
 		g_free(request->mechanism);
 		request->mechanism = g_strdup(mechusing);
 	}
-       
+
 	if (outlen > 0) {
-		outbase64 = xmlSecBase64Encode(out, outlen, 0);
+		outbase64 = xmlSecBase64Encode((xmlChar*)out, outlen, 0);
 		request->Data = g_list_append(request->Data, outbase64);
 	}
 
@@ -141,10 +139,10 @@ lasso_authentication_client_step(LassoAuthentication *authentication)
 
 	/* sasl part */
 
-	if (response->Data != NULL) {
+	if (response->Data != NULL && response->Data->data != NULL) {
 		inbase64 = response->Data->data;
-		in = g_malloc(strlen(inbase64));
-		inlen = xmlSecBase64Decode(inbase64, in, strlen(inbase64));
+		in = g_malloc(strlen((char*)inbase64));
+		inlen = xmlSecBase64Decode(inbase64, in, strlen((char*)inbase64));
 
 		res = sasl_client_step(authentication->connection, /* our context */
 				       in,    /* the data from the server */
@@ -208,16 +206,15 @@ lasso_authentication_init_request(LassoAuthentication *authentication,
 	static sasl_callback_t global_callbacks[2];
 
 	g_return_val_if_fail(LASSO_IS_AUTHENTICATION(authentication),
-			     LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
 	g_return_val_if_fail(LASSO_IS_DISCO_DESCRIPTION(description),
-			     LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(mechanisms != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(mechanisms != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 
 	if (description->Endpoint != NULL) {
 		LASSO_WSF_PROFILE(authentication)->msg_url = g_strdup(description->Endpoint);
-	}
-	else if (description->WsdlURI != NULL) {
+	} else if (description->WsdlURI != NULL) {
 
 	}
 
@@ -230,6 +227,9 @@ lasso_authentication_init_request(LassoAuthentication *authentication,
 
 	envelope = lasso_wsf_profile_build_soap_envelope(NULL, NULL);
 	LASSO_WSF_PROFILE(authentication)->soap_envelope_request = envelope;
+	if (envelope == NULL || envelope->Body == NULL || envelope->Body->any == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_MISSING_REQUEST);
+	}
 	envelope->Body->any = g_list_append(envelope->Body->any, request);
 
 	/* set up default logging callback */
@@ -298,11 +298,10 @@ lasso_authentication_process_request_msg(LassoAuthentication *authentication,
 	int res = 0;
 	
 	g_return_val_if_fail(LASSO_IS_AUTHENTICATION(authentication),
-			     LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-
+			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
 	g_return_val_if_fail(LASSO_IS_AUTHENTICATION(authentication),
-			     LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(soap_msg != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(soap_msg != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	/* if a previous request and response, then remove */
 	if (LASSO_IS_SOAP_ENVELOPE(LASSO_WSF_PROFILE(authentication)->soap_envelope_response) \
@@ -333,6 +332,9 @@ lasso_authentication_process_request_msg(LassoAuthentication *authentication,
 	status = lasso_utility_status_new(LASSO_SA_STATUS_CODE_OK);
 	response = lasso_sa_sasl_response_new(status);
 	LASSO_WSF_PROFILE(authentication)->response = LASSO_NODE(response);
+	if (envelope == NULL || envelope->Body == NULL || envelope->Body->any == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_MISSING_RESPONSE);
+	}
 	envelope->Body->any = g_list_append(envelope->Body->any, response);
 
 	/* liberty-idwsf-authn-svc-1.1.pdf - page 13 - lignes 359 / 361 :
@@ -359,8 +361,8 @@ lasso_authentication_process_response_msg(LassoAuthentication *authentication,
 	gchar *messageId;
 
 	g_return_val_if_fail(LASSO_IS_AUTHENTICATION(authentication),
-			     LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
-	g_return_val_if_fail(soap_msg != NULL, LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(soap_msg != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	/* if a previous request or response, remove */
 	if (LASSO_IS_SOAP_ENVELOPE(LASSO_WSF_PROFILE(authentication)->soap_envelope_request) \
@@ -382,9 +384,19 @@ lasso_authentication_process_response_msg(LassoAuthentication *authentication,
 	envelope = LASSO_SOAP_ENVELOPE(lasso_node_new_from_dump(soap_msg));
 	LASSO_WSF_PROFILE(authentication)->soap_envelope_response = envelope;
 
+	if (envelope == NULL || envelope->Body == NULL || envelope->Body->any == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_MISSING_RESPONSE);
+	}
 	response = envelope->Body->any->data;
+	if (response == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_MISSING_RESPONSE);
+	}
 	LASSO_WSF_PROFILE(authentication)->response = LASSO_NODE(response);
 
+	if (response->Status == NULL || response->Status->code == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_MISSING_STATUS_CODE);
+	}
+
 	/* if continue, init another request */
 	if (g_str_equal(response->Status->code, LASSO_SA_STATUS_CODE_CONTINUE) == TRUE) {
 		correlation = envelope->Header->Other->data;
@@ -483,7 +495,7 @@ lasso_authentication_server_start(LassoAuthentication *authentication)
 	}
 
 	/* decode Data if not NULL */
-	if (request->Data != NULL) {
+	if (request->Data != NULL && request->Data->data != NULL) {
 		inbase64 = request->Data->data;
 		in = g_malloc(strlen(inbase64));
 		inlen = xmlSecBase64Decode(inbase64, in, strlen(inbase64));
@@ -509,9 +521,8 @@ lasso_authentication_server_start(LassoAuthentication *authentication)
 				outbase64 = xmlSecBase64Encode(out, outlen, 0);
 				response->Data = g_list_append(response->Data, outbase64);
 			}
-		}
-		/* abort authentication */
-		else {
+		} else {
+			/* abort authentication */
 			response->Status->code = g_strdup(LASSO_SA_STATUS_CODE_ABORT);
 		}
 	}
@@ -546,7 +557,7 @@ lasso_authentication_server_step(LassoAuthentication *authentication)
 		return 0;
 	}
 
-	if (request->Data != NULL) {
+	if (request->Data != NULL && request->Data->data != NULL) {
 		inbase64 = request->Data->data;
 		in = g_malloc(strlen(inbase64));
 		inlen = xmlSecBase64Decode(inbase64, in, strlen(inbase64));
@@ -569,9 +580,8 @@ lasso_authentication_server_step(LassoAuthentication *authentication)
 				outbase64 = xmlSecBase64Encode(out, outlen, 0);
 				response->Data = g_list_append(response->Data, outbase64);
 			}
-		}
+		} else  {
 		/* authentication failed, abort exchange */
-		else  {
 			response->Status->code = g_strdup(LASSO_SA_STATUS_CODE_ABORT);
 		}
 	}
