@@ -1,4 +1,4 @@
-/* $Id: saml2_assertion.c,v 1.2 2005/11/21 18:51:52 fpeters Exp $ 
+/* $Id: saml2_assertion.c,v 1.6 2006/12/20 09:03:40 dlaniel Exp $ 
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -60,6 +60,8 @@ static struct XmlSnippet schema_snippets[] = {
 	{ "Issuer", SNIPPET_NODE,
 		G_STRUCT_OFFSET(LassoSaml2Assertion, Issuer),
 		"LassoSaml2NameID" },
+	{ "Signature", SNIPPET_SIGNATURE,
+		G_STRUCT_OFFSET(LassoSaml2Assertion, ID) },
 	{ "Subject", SNIPPET_NODE,
 		G_STRUCT_OFFSET(LassoSaml2Assertion, Subject) },
 	{ "Conditions", SNIPPET_NODE,
@@ -80,8 +82,6 @@ static struct XmlSnippet schema_snippets[] = {
 		G_STRUCT_OFFSET(LassoSaml2Assertion, ID) },
 	{ "IssueInstant", SNIPPET_ATTRIBUTE,
 		G_STRUCT_OFFSET(LassoSaml2Assertion, IssueInstant) },
-	{ "Signature", SNIPPET_SIGNATURE,
-		G_STRUCT_OFFSET(LassoSaml2Assertion, ID) },
 
 	/* hidden fields; used in lasso dumps */
 	{ "SignType", SNIPPET_ATTRIBUTE | SNIPPET_INTEGER | SNIPPET_LASSO_DUMP,
@@ -92,6 +92,12 @@ static struct XmlSnippet schema_snippets[] = {
 		G_STRUCT_OFFSET(LassoSaml2Assertion, private_key_file) },
 	{ "CertificateFile", SNIPPET_CONTENT | SNIPPET_LASSO_DUMP,
 		G_STRUCT_OFFSET(LassoSaml2Assertion, certificate_file) },
+	{ "EncryptionActivated", SNIPPET_ATTRIBUTE | SNIPPET_BOOLEAN | SNIPPET_LASSO_DUMP,
+		G_STRUCT_OFFSET(LassoSaml2Assertion, encryption_activated) },
+	{ "EncryptionPublicKeyStr", SNIPPET_CONTENT | SNIPPET_LASSO_DUMP,
+		G_STRUCT_OFFSET(LassoSaml2Assertion, encryption_public_key_str) },
+	{ "EncryptionSymKeyType", SNIPPET_ATTRIBUTE | SNIPPET_INTEGER | SNIPPET_LASSO_DUMP,
+		G_STRUCT_OFFSET(LassoSaml2Assertion, encryption_sym_key_type) },
 
 	{NULL, 0, 0}
 };
@@ -138,6 +144,11 @@ instance_init(LassoSaml2Assertion *node)
 	node->ID = NULL;
 	node->IssueInstant = NULL;
 	node->sign_type = LASSO_SIGNATURE_TYPE_NONE;
+	node->private_key_file = NULL;
+	node->certificate_file = NULL;
+	node->encryption_activated = FALSE;
+	node->encryption_public_key_str = NULL;
+	node->encryption_sym_key_type = LASSO_ENCRYPTION_SYM_KEY_TYPE_DEFAULT;
 }
 
 static void
