@@ -1,4 +1,4 @@
-/* $Id: samlp2_logout_response.c,v 1.1 2005/11/21 18:51:52 fpeters Exp $ 
+/* $Id: samlp2_logout_response.c,v 1.3 2006/12/28 14:44:56 fpeters Exp $ 
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -46,9 +46,18 @@ static gchar*
 build_query(LassoNode *node)
 {
 	char *ret, *deflated_message;
+	LassoSamlp2LogoutResponse *response = LASSO_SAMLP2_LOGOUT_RESPONSE(node);
 
 	deflated_message = lasso_node_build_deflated_query(node);
-	ret = g_strdup_printf("SAMLResponse=%s", deflated_message);
+	if (deflated_message == NULL) {
+		return NULL;
+	}
+	if (response->relayState) {
+		ret = g_strdup_printf("SAMLResponse=%s&RelayState=%s",
+				deflated_message, response->relayState);
+	} else {
+		ret = g_strdup_printf("SAMLResponse=%s", deflated_message);
+	}
 	/* XXX: must support RelayState (which profiles?) */
 	g_free(deflated_message);
 	return ret;
@@ -75,6 +84,7 @@ init_from_query(LassoNode *node, char **query_fields)
 static void
 instance_init(LassoSamlp2LogoutResponse *node)
 {
+	node->relayState = NULL; /* XXX: free me sometimes */
 }
 
 static void
