@@ -1,4 +1,4 @@
-/* $Id: strings.h,v 1.54 2005/11/15 11:14:30 nclapies Exp $ 
+/* $Id: strings.h,v 1.61 2006/11/27 11:05:27 dlaniel Exp $ 
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -292,6 +292,8 @@
 #define LASSO_SOAP_ENV_HREF   "http://schemas.xmlsoap.org/soap/envelope/"
 #define LASSO_SOAP_ENV_PREFIX "s"
 
+#define LASSO_SOAP_ENV_ACTOR "http://schemas.xmlsoap.org/soap/actor/next"
+
 #define LASSO_SOAP_BINDING_HREF   "urn:liberty:sb:2003-08"
 #define LASSO_SOAP_BINDING_PREFIX "sb"
 
@@ -308,16 +310,31 @@
 #define LASSO_SOAP_BINDING_PROCESS_CONTEXT_SIMULATE \
 	"urn:liberty:sb:2003-08:ProcessingContext:Simulate"
 
+/*****************************************************************************/
+/* POAS BINDING                                                              */
+/*****************************************************************************/
+
+#define LASSO_PAOS_HREF   "urn:liberty:paos:2003-08"
+#define LASSO_PAOS_PREFIX "paos"
+
+/*****************************************************************************/
+/* ECP BINDING                                                              */
+/*****************************************************************************/
+
+#define LASSO_ECP_HREF   "urn:oasis:names:tc:SAML:2.0:profiles:SSO:ecp"
+#define LASSO_ECP_PREFIX "ecp"
+
 /*****************************************************************************/
 /* SAML 2.0                                                                  */
 /*****************************************************************************/
 
-#define LASSO_SAML20_METADATA_HREF "urn:oasis:names:tc:SAML:2.0:metadata"
+#define LASSO_SAML2_METADATA_HREF "urn:oasis:names:tc:SAML:2.0:metadata"
 
-#define LASSO_SAML20_METADATA_BINDING_SOAP "urn:oasis:names:tc:SAML:2.0:bindings:SOAP"
-#define LASSO_SAML20_METADATA_BINDING_REDIRECT "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
-#define LASSO_SAML20_METADATA_BINDING_POST "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
-#define LASSO_SAML20_METADATA_BINDING_ARTIFACT "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Artifact"
+#define LASSO_SAML2_METADATA_BINDING_SOAP "urn:oasis:names:tc:SAML:2.0:bindings:SOAP"
+#define LASSO_SAML2_METADATA_BINDING_REDIRECT "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
+#define LASSO_SAML2_METADATA_BINDING_POST "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
+#define LASSO_SAML2_METADATA_BINDING_ARTIFACT "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Artifact"
+#define LASSO_SAML2_METADATA_BINDING_PAOS "urn:oasis:names:tc:SAML:2.0:bindings:PAOS"
 
 #define LASSO_SAML2_PROTOCOL_HREF "urn:oasis:names:tc:SAML:2.0:protocol"
 #define LASSO_SAML2_PROTOCOL_PREFIX "samlp"
@@ -339,6 +356,8 @@
 		"urn:oasis:names:tc:SAML:2.0:nameid-format:persistent"
 #define LASSO_SAML2_NAME_IDENTIFIER_FORMAT_TRANSIENT \
 		"urn:oasis:names:tc:SAML:2.0:nameid-format:transient"
+#define LASSO_SAML2_NAME_IDENTIFIER_FORMAT_ENCRYPTED \
+		"urn:oasis:names:tc:SAML:2.0:nameid-format:encrypted"
 
 
 /* Consent */
@@ -393,8 +412,60 @@
 #define LASSO_SAML2_STATUS_CODE_UNSUPPORTED_BINDING \
 		"urn:oasis:names:tc:SAML:2.0:status:UnsupportedBinding"
 
+/* AuthnClassRef */
 
+#define LASSO_SAML2_AUTHN_CONTEXT_AUTHENTICATED_TELEPHONY \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:AuthenticatedTelephony"
+#define LASSO_SAML2_AUTHN_CONTEXT_INTERNET_PROTOCOL \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:InternetProtocol"
+#define LASSO_SAML2_AUTHN_CONTEXT_INTERNET_PROTOCOL_PASSWORD \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:InternetProtocolPassword"
+#define LASSO_SAML2_AUTHN_CONTEXT_KERBEROS \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:Kerberos"
+#define LASSO_SAML2_AUTHN_CONTEXT_MOBILE_ONE_FACTOR_CONTRACT \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:MobileOneFactorContract"
+#define LASSO_SAML2_AUTHN_CONTEXT_MOBILE_ONE_FACTOR_UNREGISTERED \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:MobileOneFactorUnregistered"
+#define LASSO_SAML2_AUTHN_CONTEXT_MOBILE_TWO_FACTOR_CONTRACT \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:MobileTwoFactorContract"
+#define LASSO_SAML2_AUTHN_CONTEXT_MOBILE_TWO_FACTOR_UNREGISTERED \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:MobileTwoFactorUnregistered"
+#define LASSO_SAML2_AUTHN_CONTEXT_NOMAD_TELEPHONY \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:NomadTelephony"
+#define LASSO_SAML2_AUTHN_CONTEXT_PERSONALIZED_TELEPHONY \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:PersonalizedTelephony"
+#define LASSO_SAML2_AUTHN_CONTEXT_PGP \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:PGP"
+#define LASSO_SAML2_AUTHN_CONTEXT_PASSWORD_PROTECTED_TRANSPORT \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport"
+#define LASSO_SAML2_AUTHN_CONTEXT_PASSWORD \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:Password"
+#define LASSO_SAML2_AUTHN_CONTEXT_PREVIOUS_SESSION \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:PreviousSession"
+#define LASSO_SAML2_AUTHN_CONTEXT_SMARTCARD \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:Smartcard"
+#define LASSO_SAML2_AUTHN_CONTEXT_SMARTCARD_PKI \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:SmartcardPKI"
+#define LASSO_SAML2_AUTHN_CONTEXT_SOFTWARE_PKI \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:SoftwarePKI"
+#define LASSO_SAML2_AUTHN_CONTEXT_SPKI \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:SPKI"
+#define LASSO_SAML2_AUTHN_CONTEXT_SECURE_REMOTE_PASSWORD \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:SecureRemotePassword"
+#define LASSO_SAML2_AUTHN_CONTEXT_TLS_CLIENT \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:TLSClient"
+#define LASSO_SAML2_AUTHN_CONTEXT_X509 \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:X509"
+#define LASSO_SAML2_AUTHN_CONTEXT_TELEPHONY \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:Telephony"
+#define LASSO_SAML2_AUTHN_CONTEXT_TIME_SYNC_TOKEN \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:TimeSyncToken"
+#define LASSO_SAML2_AUTHN_CONTEXT_XMLDSIG \
+	"urn:oasis:names:tc:SAML:2.0:ac:classes:XMLDSig"
 
+/* Confirmation methods */
+
+#define LASSO_SAML2_CONFIRMATION_METHOD_BEARER "urn:oasis:names:tc:SAML:2.0:cm:bearer"
 
 /*****************************************************************************/
 /* Others                                                                    */
