@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# $Id: patch_swig_output.py,v 1.15 2006/03/20 19:26:21 fpeters Exp $
+# $Id: patch_swig_output.py,v 1.18 2006/12/28 15:42:17 fpeters Exp $
 #
 # SWIG based PHP binding for Lasso Library
 #
@@ -125,7 +125,7 @@ with:
         *return_value=*obj;
     }}
 
-This program corrects (3), by replacing things like:
+In old SWIG versions, this program corrects (3), by replacing things like:
     if(zend_get_parameters_array_ex(arg_count-argbase,args)!=SUCCESS)
 with:
     if(zend_get_parameters_array_ex(arg_count,args)!=SUCCESS)
@@ -133,6 +133,12 @@ and by replacing:
     if(arg_count > 1) {
 with:
     if(arg_count > 1 - argbase) {
+
+In newer SWIG versions, this program corrects (3), by replacing code like:
+    if(arg_count<2 || arg_count>4)
+with:
+    if(arg_count<1 || arg_count>3)
+whenever the function uses a this_ptr.
 """
 
 import re
@@ -173,7 +179,7 @@ end = """
 i = wrap.find(begin)
 while i >= 0:
     j = wrap.find(end, i) + len(end)
-    print >> sys.stderr, "END:", j, len(end)
+    #print >> sys.stderr, "END:", j, len(end)
     if j < len(end): # bails out if not found
         break
     segment = wrap[i:j]
@@ -188,14 +194,44 @@ wrap = wrap.replace('if(zend_get_parameters_array_ex(arg_count-argbase,args)!=SU
                     'if(zend_get_parameters_array_ex(arg_count,args)!=SUCCESS)')
 
 
+function_pattern = re.compile('ZEND_NAMED_FUNCTION(.*?)\n}', re.DOTALL)
+argcount_less_pattern = re.compile('if\(arg_count<(\d) \|\| arg_count>(\d)')
+argcount_more_pattern = re.compile('if\(arg_count > (\d)\)')
+
+
+def rep2(match):
+    arg1 = int(match.group(1)) - 1
+    arg2 = int(match.group(2)) - 1
+    return 'if(arg_count<%s || arg_count>%s' % (arg1, arg2)
+
+def rep3(match):
+    arg1 = int(match.group(1)) - 1
+    return 'if(arg_count > %s)' % arg1
+
+def rep(match):
+    m = match.group(0)
+    if not 'This function uses a this_ptr' in m:
+        return m
+    if not 'arg_count<' in m:
+        return m
+    lines = match.group(0).splitlines()
+    s = []
+    for l in lines:
+        if l.startswith('if(arg_count<'):
+            l = argcount_less_pattern.sub(rep2, l)
+        elif l.startswith('  if(arg_count >'):
+            l = argcount_more_pattern.sub(rep3, l)
+        s.append(l)
+
+    return ''.join(s)
+
+wrap = function_pattern.sub(rep, wrap)
+
 wrap = re.sub(r'zend_register_internal_class_ex(.*)NULL,NULL\)',
     r'zend_register_internal_class_ex\1NULL,NULL TSRMLS_CC)',  wrap)
 
 wrap = re.sub('zend_rsrc_list_get_rsrc_type(.*)lval',
     r'zend_rsrc_list_get_rsrc_type\1lval TSRMLS_CC', wrap)
 
-wrap = wrap.replace('zval *return_value=&_return_value;',
-    'zval *return_value=&_return_value;\n    TSRMLS_FETCH();\n')
-
 
 print wrap
